
##
## Supporting Information - R script for the manuscript paper entitled:
## "Systematic site selection for multi-species monitoring networks"
## Carvalho, S.[1], Gonalves, J.[1], Guisan, A.[2], Honrado, J.[1], 2015
## Journal Of Applied Ecology
## 
## [1] InBIO/CIBIO - Research Center in Biodiversity and Genetic Resources
## 	   Faculty of Sciences - University of Porto
## [2] University of Lausanne, Ecology and Evolution
## 
## Contacting author on this section: 
## Joo Gonalves (jfgoncalves[at]fc.up.pt / joaofgo[at]gmail.com)
##
## Script title: auxiliary functions for processing, sampling and calculation
##


## ------------------------------
## Declare some useful functions
## ------------------------------


countIf<-function(x,cond=">0"){
	return(length(x[eval(parse(text=paste("x",cond)))]))
}

## Conditional concatenate and print

condCat<-function(...,cond){
	if(cond){
		cat(...)	
	}
}

## Pads a given number with a character (0 by default)
## Used to allow correct filename sorting   

pad.number<-function(x,max.val,pad="0"){
	pad.it<-function(x,max.val=max.val,pad=pad) paste(paste(rep(pad,times=(nchar(format(max.val, scientific=FALSE))-nchar(format(x, scientific=FALSE)))),collapse=""),x,sep="")
	sapply(x,pad.it,max.val=max.val,pad=pad)
}

## Determines and (randomly) adjusts the number of samples per stratum 
##
## propStrata	Proportion of each stratum in terms of area or frequency
## n			Total number of samples

numSampPerStrata<-function(propStrata,n){
	
	if(!inherits(propStrata,"table"))
		stop("propStrata must be an object of class table")
	if((1-sum(propStrata)) > 1E-5)
		stop("sum(propStrata) must be equal to 1")
	
	nSamp<-round(propStrata*n)
	diffN<-n-sum(nSamp)
	
	if(diffN==0){
		return(nSamp)
	}else if(diffN>0){
		##
		#  The number of samples is less than the target	
		for(i in 1:diffN){
			selS<-sample(1:length(propStrata),1)
			nSamp[selS]<-nSamp[selS]+1
		}
		return(nSamp)
	}else{
		##
		#  The number of samples is greater than the target
		repeat{
			selS<-sample(1:length(propStrata),1)
			if(nSamp[selS]!=0){
				nSamp[selS]<-nSamp[selS]-1
			}else{
				next
			}
			
			if((n-sum(nSamp))==0){
				break
			}
		}
		return(nSamp)
	}
}	

##
## A variation of the original function that incorporates a minimum number of samples for all strata
## Setting this minimum value is considered the initial step after which the sample size 
## nTotal = [nTotal - (nMin * nStrata)]

numSampPerStrata1<-function(propStrata,nTotal,nMin=NULL){
	
	if(!inherits(propStrata,"table"))
		stop("propStrata must be an object of class table")
	if((1-sum(propStrata)) > 1E-5)
		stop("sum(propStrata) must be equal to 1")
	

	if(!is.null(nMin)){
		nStrata<-length(propStrata)
		if((nStrata*nMin) > nTotal){
			stop("The sample size is less than nStrata * nMin!")
		}else{
			nSamp<-rep(nMin,each=nStrata)
			
			nTotal_i<-nTotal-(nStrata*nMin)
			nSamp<-nSamp+round(propStrata*nTotal_i)
		}
	}else{
		nSamp<-round(propStrata*nTotal)
	}
	
	# Check the number of samples and randomly adjusts it
	diffN<-nTotal-sum(nSamp)
	
	if(diffN==0){
		names(nSamp)<-names(propStrata)
		return(as.table(nSamp))
	}else if(diffN>0){
		##
		#  The number of samples is less than the target	
		for(i in 1:diffN){
			selS<-sample(1:length(propStrata),1)
			nSamp[selS]<-nSamp[selS]+1
		}
		names(nSamp)<-names(propStrata)
		return(nSamp)
	}else{
		##
		#  The number of samples is greater than the target
		repeat{
			selS<-sample(1:length(propStrata),1)
			if(nSamp[selS]!=0){
				nSamp[selS]<-nSamp[selS]-1
			}else{
				next
			}
			
			if((nTotal-sum(nSamp))==0){
				break
			}
		}
		names(nSamp)<-names(propStrata)
		return(as.table(nSamp))
	}

}


## Performs [St]ratified [R]andom [S]ampling
##
## x 		Vector of indices
## strata	Character vector containing the stratum for each element of x
##			Unique elements in strata must equal the names property of nsps
## nsps		An object of class table with indicating the number of samples per stratum
## verbose	Throws an warning if the sample size is higher than the number of sample units
##
##	This version of the function automatically adjusts the sample size to the minimum number 
StRS<-function(x,strata,nsps,verbose=TRUE){
	
	if(!inherits(nsps,"table"))
		stop("nsps must be an object of class table!")
	if(!inherits(strata,"character"))
		stop("strata must be a vector of class character!")
	if(length(strata)!=length(x))
		stop("strata and x must have the same vector length!")
	
	samples<-vector()
	strataNames<-names(nsps)
	
	# Get the number of sample units per stratum
	lenStrata<-aggregate(x,list(strata),length)
	
	for(sn in strataNames){
		n<-nsps[sn]
		
		if(n!=0){
						
			## Adjust sample size
			if(lenStrata[lenStrata[,"Group.1"]==sn,2] < n){
				n_old<-n
				n<-lenStrata[lenStrata[,"Group.1"]==sn,2]
				
				if(verbose)
					warning("The sample size for stratum ",sn," was adjusted (",n_old," -> ",lenStrata[lenStrata[,"Group.1"]==sn,2],") to the number of sampleable units!")
			}
			
			samples<-c(samples,sample(x[strata==sn],n))	
		}else{
			next
		}
	}
	return(samples)
}


# Calculates the difference between extreme values
range.diff<-function(x) range(x)[2]-range(x)[1]


# Convert a list containing two levels into a data frame
list2df<-function(x,scn,targets,nRep){
	i<-0
	for(s in scn){
		for(target in targets){
			
			i<-i+1
			if(i==1){
				
				tmp.df<-data.frame(Ind=x[[s]][[target]],scenario=s,target=target,rep=1:nRep)
				
			}else{
				tmp.df<-rbind(tmp.df,data.frame(Ind=x[[s]][[target]],scenario=s,target=target,rep=1:nRep))
			}
		}
	}
	return(tmp.df)
}


## Target accomplishment
##

targetAccomp<-function(ni,N,target){
	out<-vector("integer",length(ni))
	# Target is attainable and is achieved
	out[N>=target & ni>=target]<-1
	# Target is unattainable and all possible samples are selected
	out[target>N & ni>=N]<-1
	return(countIf(out,">=1")/length(ni))
}




