
##
## Supporting Information - R script for the manuscript paper entitled:
## "Systematic site selection for multi-species monitoring networks"
## Carvalho, S.[1], Gonalves, J.[1], Guisan, A.[2], Honrado, J.[1], 2015
## Journal Of Applied Ecology
## 
## [1] InBIO/CIBIO - Research Center in Biodiversity and Genetic Resources
## 	   Faculty of Sciences - University of Porto
## [2] University of Lausanne, Ecology and Evolution
## 
## Contacting author on this section: 
## Joo Gonalves (jfgoncalves[at]fc.up.pt / joaofgo[at]gmail.com)
##
## Script title: calculate indicators by monitoring network simulated by optimized
## and non-optimized schemes
##

require(vegan)
require(foreign)
setwd("D:/MyDocs/OptimMonitNetworks")

## Load auxiliary functions (for sampling)
source("D:/MyDocs/OptimMonitNetworks/AuxiliaryFunctions.R")

## Load the objects containing the number of samples and the 
## sampled units per target/scenario/replicate
load("optimSampCost.Nsamp.RData")
load("optimSampCost.Units.RData")
load("nonOptimSamp_WithCost.RData")
load("optimSampCost_AllObjectsIndices.RData")
load("MonintNetworkCompObjs.RData")

## Load external data for AP, Strata, Species Richness and average survey effort cost
dataDF<-read.dbf("D:/MyDocs/OptimMonitNetworks/RepAnf_SDMs_UTM10km_v04.dbf",as.is=TRUE)

rid<-1:nrow(dataDF) 		# Row integer ids (for sampling purposes)
nRep<-100					# Number of replicates


## -------------------------------------------------------------------------- ##
## Define some parameters and data specifications
##
ID<-"ID"   				    # Column with row IDs (MGRS codes)
UID<-"UNIT_ID" 			    # Column with unique row IDs
SpCols<-5:49				# Species occurence/distribution by 10km quadrats
Strata<-50:65				# Strata

Stratum<-66					# Dominant stratum in the quadrat
StratumCname<-"MAJORITY"	# Dominant stratum in the quadrat (column name)

PAs<-68						# Protected areas
PAsCName<-"APS"				# Protected areas (column name)

SpRich<-69					# Specie richness
SpRich<-"SpRich"			# Species richness (column name)

cost<-71
costCname<-"MEAN"

## -------------------------------------------------------------------------- ##


## Pseudo-real prevalence
np_i<-apply(dataDF[,SpCols],2,sum)
#na_i<-nrow(dataDF)-np_i
#prev_i<-np_i/na_i
prev_i<-np_i/nrow(dataDF)

## Scenarios and targets
scn<-c("SP","SP_PA","SP_ST","SP_ST_PA")
scn.new<-c("NS", "PA", "ENV", "ENVPA")
targets<-c(paste("T",seq(10,50,10),sep=""))
targNum<-seq(10,50,10)
names(targNum)<-targets
verbose<-TRUE


### Define indices -------------------------------------------------------------------------

Indices<-list(
  
  ind_abbr=c(  
    "wmsp",
    "mnsp", 
    "ind1", 
    "targAcc",
    "msr",		
    "shdiv",
    "nstrata"
  ),
  
  ind_name=c(
    "Rarity-weighted mean number of species occurrences (RWMSO)",
    "Mean number of species presences",
    "Index_1",
    "Percentage of species attaining the specified target",
    "Mean species richness",
    "Species diversity and evenness (H')",
    "Number of environmental strata (Nenv)"
  ),
  
  func_def_ops=c(
    "weighted.mean(ops.npps_i,(1/prev_i))",
    "mean(ops.npps_i)",
    "(1/nS)*(sum((1/prev_i)*ops.npps_i))",
    "targetAccomp(ops.npps_i,np_i,targNum[target])",
    "mean(dataDF[rid[ops.rids],SpRich])",
    "diversity(apply(dataDF[rid[ops.rids],SpCols],2,sum),\"shannon\")",
    "length(unique(dataDF[rid[ops.rids],StratumCname]))"
  ),
  
  func_def_nos=c(
    "weighted.mean(nos.npps_i,(1/prev_i))",
    "mean(nos.npps_i)",
    "(1/nS)*(sum((1/prev_i)*nos.npps_i))",
    "targetAccomp(nos.npps_i,np_i,targNum[target])",
    "mean(dataDF[rid[nos.rids],SpRich])",
    "diversity(apply(dataDF[rid[nos.rids],SpCols],2,sum),\"shannon\")",
    "length(unique(dataDF[rid[nos.rids],StratumCname]))"
  ),
  
  lab_height=c(
    30.5,		# wmsp
    105,	# mnsp
    103,	# ind1
    0.73,	# targAcc
    30.2,	# msr	
    3.64,	# shdiv
    14		# nstrata
  ),
  
  axis_at=list(
    seq(5,30,5),		# wmsp
    seq(20,100,10),		# mnsp
    seq(40,100,10),		# ind1
    seq(0.65,1,0.05),	# targAcc
    seq(22,30,2),		# msr	
    seq(3.6,3.76,0.02),	# shdiv
    seq(4,14,2)			# nstrata
  )
  
)

### Start lists ------------------------------------------------------------------------------------
## Start the lists that will hold data generated by each of the indices calculated

optimSamp.NPPS<-list()
nonOptimSamp.NPPS<-list()

for(k in 1:length(Indices[[1]])){
	
	eval(parse(text=paste("optimSamp.Ind",k,"<-list()",sep="")))
	eval(parse(text=paste("nonOptimSamp.Ind",k,"<-list()",sep="")))
}


### Calculate indices -------------------------------------------------------------------------------

for(s in scn){
	for(target in targets){
		
		# Subselect number of samples that correspond to the scenario/target
		nSamp<-optimSamp.Nsamp[(optimSamp.Nsamp[,"Scenario"]==s & optimSamp.Nsamp[,"Target"]==target),]
		
		# Start data holders
		ops.npps<-matrix(0,ncol=length(SpCols),nrow=nRep,dimnames=list(1:nRep,colnames(dataDF)[SpCols]))
		nos.npps<-matrix(0,ncol=length(SpCols),nrow=nRep,dimnames=list(1:nRep,colnames(dataDF)[SpCols]))
		
		
		# Start temp data holders for each index
		for(k in 1:length(Indices[[1]])){
			
			eval(parse(text=paste("ops.Ind",k,"<-vector(length=nRep,mode=\"numeric\")",sep="")))
			eval(parse(text=paste("nos.Ind",k,"<-vector(length=nRep,mode=\"numeric\")",sep="")))
		}
		
		for(i in 3:(nRep+2)){ ## the first 2 columns are related to ids  
			
			# Number of samples
			nS<-nSamp[i-2,"NSamples"]
			
			# Generic structs - OPS
			#ops.rids<-as.logical(optimSamp.Units[[target]][[s]][,i])
			
			ops.rids<-dataDF[,UID] %in% (1:nrow(optimSamp.Units[[target]][[s]]))[as.logical(optimSamp.Units[[target]][[s]][,i])]
			ops.npps_i<-apply(dataDF[rid[ops.rids],SpCols],2,sum)
			ops.npps[i-2,]<-ops.npps_i
			
			# Generic structs - NOS
			nos.rids<-as.logical(nonOptimSamp.Units[[s]][[target]][,i])
			nos.npps_i<-apply(dataDF[rid[nos.rids],SpCols],2,sum)
			nos.npps[i-2,]<-nos.npps_i
			
			# Evaluate the index functions at network level
			for(k in 1:length(Indices[[1]])){
				
				eval(parse(text=paste("ops.Ind",k,"[i-2]<-",Indices[["func_def_ops"]][k],sep="")))
				eval(parse(text=paste("nos.Ind",k,"[i-2]<-",Indices[["func_def_nos"]][k],sep="")))
			}
		}
		
		# Number of times the species was sampled for the combo scn/tgt/rep
		optimSamp.NPPS[[s]][[target]]<-ops.npps
		nonOptimSamp.NPPS[[s]][[target]]<-nos.npps
		
		# Insert data per scenario/target
		for(k in 1:length(Indices[[1]])){
			
			eval(parse(text=paste("optimSamp.Ind",k,"[[s]][[target]]<-ops.Ind",k,sep="")))
			eval(parse(text=paste("nonOptimSamp.Ind",k,"[[s]][[target]]<-nos.Ind",k,sep="")))
		}		
	}
}

### Make boxplots ---------------------------------------------------------------------------------------

# restrict targets
targets<-c("T10","T30","T50")

c1<-colors()[240] # NOS
c2<-colors()[190] # OPS

for(k in 1:length(Indices[[1]])){
	
	cat("Box-Plotting index...",Indices[["ind_abbr"]][k])
	
	eval(parse(text=paste("DF.IND",k,".ops<-list2df(optimSamp.Ind",k,",scn,targets,nRep)",sep="")))
	eval(parse(text=paste("DF.IND",k,".nos<-list2df(nonOptimSamp.Ind",k,",scn,targets,nRep)",sep="")))
	eval(parse(text=paste("DF.IND",k,"<-cbind(rbind(DF.IND",k,".ops,DF.IND",k,".nos),sampType=rep(c(\"OPS\",\"NOS\"),each=nrow(DF.IND",k,".ops)))",sep="")))

## --------------------------------------------------------------------------- ##
# Removed T20 and T40
## --------------------------------------------------------------------------- ##

bxplot<-paste("png(filename=\"D:/MyDocs/Projects/Paper_OptimMonitNetworks_SilviaC/PLOTS/Boxplots_v2/",Indices[["ind_abbr"]][k],".png\",4000,3000,res=300)	  
          #par(cex=0.7)
          par(mar=c(6,5,2,2))
          boxplot(Ind~sampType+scenario+target,data=DF.IND",k,",boxwex=0.7,las=2,names=rep(scn.new,3,each=2), 
							cex.axis=1.4, cex.lab=1.4, axes = FALSE,
							col=rep(rep(c(c1,c2),4),5),ylab=\"",Indices[["ind_name"]][k],"\")
					
					axis(side = 1, at = 1:24, cex.axis=1.4, labels=rep(scn.new,3,each=2),las=2)
					axis(side = 2, at = ",Indices[["axis_at"]][k],",cex.axis=1.4)
					box()

					for(j in c(8.5,16.5)) abline(v=j,lty=3,col=\"black\")
					
					legend(\"bottomright\",legend=c(\"Non-optimized (NOPT)\",\"Optimized (OPT)\"),fill=c(c1,c2),col=c(c1,c2),cex=1.4)
					text(x=c(4.5,12.5,20.5,28.5,36.5),y=",Indices[["lab_height"]][k],",labels=targets,cex=1.4)
					dev.off()",sep="")

	eval(parse(text=paste(bxplot,sep="")))	
	cat(" .... done.\n")
	
}

