
##
## Supporting Information - R script for the manuscript paper entitled:
## "Systematic site selection for multi-species monitoring networks"
## Carvalho, S.[1], Gonalves, J.[1], Guisan, A.[2], Honrado, J.[1], 2015
## Journal Of Applied Ecology
## 
## [1] InBIO/CIBIO - Research Center in Biodiversity and Genetic Resources
## 	   Faculty of Sciences - University of Porto
## [2] University of Lausanne, Ecology and Evolution
## 
## Contacting author on this section: 
## Joo Gonalves (jfgoncalves[at]fc.up.pt / joaofgo[at]gmail.com)
##
## Script title: simulate non-optimized networks based on SRS and StRS
##

require(foreign)

setwd("D:/MyDocs/OptimMonitNetworks")

## Load auxiliar functions (for sampling)
source("D:/MyDocs/OptimMonitNetworks/AuxiliaryFunctions.R")

## Load the objects containing the number of samples and the 
## sampled units per target/scenario/replicate
load("optimSampCost.Nsamp.RData")
load("optimSampCost.Units.RData")

## Read data
dataDF<-read.dbf("D:/MyDocs/OptimMonitNetworks/RepAnf_SDMs_UTM10km_v04.dbf",as.is=TRUE)


## Define some parameters and data specifications
##
ID<-"ID" 					    # Column with row IDs (MGRS codes)
UID<-"UNIT_ID" 				    # Column with unique row IDs
SpCols<-7:51				    # Species occurrence/distribution by 10km quadrats
Strata<-52:67				    # Strata

Stratum<-68					    # Dominant stratum in the quadrat
StratumCname<-"MAJORITY"	    # Dominant stratum in the quadrat (column name)

PAs<-70						    # Protected areas
PAsCName<-"APS"				    # Protected areas (column name)

SpRich<-71					    # Specie richness
SpRich<-"SpRich"			    # Species richness (column name)

rid<-1:nrow(dataDF) 		    # Row integer ids (for sampling purposes)
nRep<-100					    # Number of replicates

## Scenarios and targets
scn<-c("SP","SP_PA","SP_ST","SP_ST_PA")
targets<-c(paste("T",seq(10,50,10),sep=""))
verbose<-TRUE


## -------------- ##
## Data analysis
## -------------- ##

nonOptimSamp.Units<-list()


## -----------------------------------------------------------------------
## SP (species-only) scenario -> uses simple random sampling to simulate 
## non-optimized alternative networks (for horizontal comparison purposes)
## -----------------------------------------------------------------------

activeScenario<-scn[1]

for(target in targets){
	
	nonOptimSamp.Units[[activeScenario]][[target]]<-as.data.frame(matrix(0,nrow=nrow(dataDF),ncol=nRep+2,
			dimnames=list(c(1:nrow(dataDF)),c("ID","UTM10K",paste(target,"SP",1:nRep,sep="_")))))

	nonOptimSamp.Units[[activeScenario]][[target]][,1:2]<-dataDF[,c(ID,UID)]
	nSamp<-optimSamp.Nsamp[(optimSamp.Nsamp[,"Scenario"]==activeScenario & optimSamp.Nsamp[,"Target"]==target),]
	
	for(i in 1:nRep){
		
		condCat(activeScenario,target,i,nSamp[i,"NSamples"],"\n",sep=" | ",cond=verbose)
		
		sampInd<-sample(rid,nSamp[i,"NSamples"])
		nonOptimSamp.Units[[activeScenario]][[target]][sampInd,i+2]<-1	
	}
}


## --------------------------------------------------------------------------------------------
## SP_PA (species and protected areas) scenario -> uses stratified random sampling to simulate 
## non-optimized alternative networks (for horizontal comparison purposes)
## --------------------------------------------------------------------------------------------

activeScenario<-scn[2]

# Proportion of units per stratum
#prop.strata<-table(dataDF[,PAs])/nrow(dataDF)

# Modified on 23.05.2012 based on a suggestion from Slvia C
prop.strata<-as.table(c(0.5,0.5))
names(prop.strata)<-c("0","1")

for(target in targets){
	
	nonOptimSamp.Units[[activeScenario]][[target]]<-as.data.frame(matrix(0,nrow=nrow(dataDF),ncol=nRep+2,
					dimnames=list(c(1:nrow(dataDF)),c("ID","UTM10K",paste(target,"SP",1:nRep,sep="_")))))
	
	nonOptimSamp.Units[[activeScenario]][[target]][,1:2]<-dataDF[,c(ID,UID)]
	nSamp<-optimSamp.Nsamp[(optimSamp.Nsamp[,"Scenario"]==activeScenario & optimSamp.Nsamp[,"Target"]==target),]
	
	for(i in 1:nRep){
		
		condCat(activeScenario,target,i,nSamp[i,"NSamples"],"\n",sep=" | ",cond=verbose)
		
		nSampPerStratum<-numSampPerStrata(prop.strata,nSamp[i,"NSamples"])
		sampInd<-StRS(rid,as.character(dataDF[,PAs]),nSampPerStratum)
		nonOptimSamp.Units[[activeScenario]][[target]][sampInd,i+2]<-1
		
	}
}


## --------------------------------------------------------------------------------------------
## SP_ST (species and environmental stratification) scenario -> uses stratified random 
## sampling to simulate non-optimized alternative networks (for horizontal comparison purposes)
## --------------------------------------------------------------------------------------------

activeScenario<-scn[3]

# Proportion of units per stratum
prop.strata<-table(dataDF[,Stratum])/nrow(dataDF)

for(target in targets){
	
	nonOptimSamp.Units[[activeScenario]][[target]]<-as.data.frame(matrix(0,nrow=nrow(dataDF),ncol=nRep+2,
					dimnames=list(c(1:nrow(dataDF)),c("ID","UTM10K",paste(target,"SP",1:nRep,sep="_")))))
	
	nonOptimSamp.Units[[activeScenario]][[target]][,1:2]<-dataDF[,c(ID,UID)]
	nSamp<-optimSamp.Nsamp[(optimSamp.Nsamp[,"Scenario"]==activeScenario & optimSamp.Nsamp[,"Target"]==target),]
	
	for(i in 1:nRep){
		
		condCat(activeScenario,target,i,nSamp[i,"NSamples"],"\n",sep=" | ",cond=verbose)
		
		nSampPerStratum<-numSampPerStrata(prop.strata,nSamp[i,"NSamples"])
		#print(nSampPerStratum)
		
		sampInd<-StRS(rid,as.character(dataDF[,Stratum]),nSampPerStratum)
		nonOptimSamp.Units[[activeScenario]][[target]][sampInd,i+2]<-1
		
	}
}


## ---------------------------------------------------------------------------------------------------------
## SP_ST_PA (species, environmental stratification, and protected areas) scenario -> uses stratified random 
## sampling to simulate non-optimized alternative networks (for horizontal comparison purposes)
## ---------------------------------------------------------------------------------------------------------

activeScenario<-scn[4]

# Generate a new strata layer based on the combination of 
# environmental strata and protected areas 
newStrata<-paste(dataDF[,Stratum],dataDF[,PAs],sep="_")
# Proportion of units per stratum
prop.strata<-table(newStrata)/nrow(dataDF)

for(target in targets){
	
	nonOptimSamp.Units[[activeScenario]][[target]]<-as.data.frame(matrix(0,nrow=nrow(dataDF),ncol=nRep+2,
					dimnames=list(c(1:nrow(dataDF)),c("ID","UTM10K",paste(target,"SP",1:nRep,sep="_")))))
	
	nonOptimSamp.Units[[activeScenario]][[target]][,1:2]<-dataDF[,c(ID,UID)]
	nSamp<-optimSamp.Nsamp[(optimSamp.Nsamp[,"Scenario"]==activeScenario & optimSamp.Nsamp[,"Target"]==target),]
	
	for(i in 1:nRep){
		
		condCat(activeScenario,target,i,nSamp[i,"NSamples"],"\n",sep=" | ",cond=verbose)
		
		nSampPerStratum<-numSampPerStrata(prop.strata,nSamp[i,"NSamples"])
		#print(nSampPerStratum)
		
		sampInd<-StRS(rid,as.character(newStrata),nSampPerStratum)
		nonOptimSamp.Units[[activeScenario]][[target]][sampInd,i+2]<-1
		
	}
}

save.image(file="D:/MyDocs/OptimMonitNetworks/nonOptimSamp_WithCost.RData")


